{% extends "kleides_mfa/base.html" %}

{% load i18n %}

{% block content %}
<h1>{% blocktrans with name=plugin.name %}Update {{ name }} authentication{% endblocktrans %}</h1>

{% block form_description %}{% endblock form_description %}

<form id="kleides-mfa-form" action="" method="post" novalidate>{% csrf_token %}
    {% include "kleides_mfa/form.html" %}

    <button class='btn btn-primary' type='submit'>{% trans 'Update' %}</button>
</form>
{% endblock content %}
